@@ -5,12 +5,18 @@
 
 from typing import Any, Dict, Optional
 
-from streaming.base.format.base.reader import Reader
+from streaming.base.format.base import FileInfo, Reader
+from streaming.base.format.index import get_index_basename
 from streaming.base.format.json import JSONReader, JSONWriter
 from streaming.base.format.mds import MDSReader, MDSWriter
 from streaming.base.format.xsv import (CSVReader, CSVWriter, TSVReader, TSVWriter, XSVReader,
                                        XSVWriter)
 
+__all__ = [
+    'CSVWriter', 'FileInfo', 'get_index_basename', 'JSONWriter', 'MDSWriter', 'Reader',
+    'reader_from_json', 'TSVWriter', 'XSVWriter'
+]
+
 _readers = {
     'csv': CSVReader,
     'json': JSONReader,
@@ -34,6 +40,3 @@ def reader_from_json(dirname: str, split: Optional[str], obj: Dict[str, Any]) ->
     assert obj['version'] == 2
     cls = _readers[obj['format']]
     return cls.from_json(dirname, split, obj)
-
-
-__all__ = ['CSVWriter', 'JSONWriter', 'MDSWriter', 'reader_from_json', 'TSVWriter', 'XSVWriter']
