@@ -7,17 +7,22 @@ import json
 import logging
 import os
 import shutil
+import sys
 from abc import ABC, abstractmethod
 from concurrent.futures import ThreadPoolExecutor
+from concurrent.futures._base import Future
+from threading import Event
+from time import sleep
 from types import TracebackType
 from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
 from typing_extensions import Self
 
 from streaming.base.compression import compress, get_compression_extension, is_compression
+from streaming.base.format.index import get_index_basename
 from streaming.base.hashing import get_hash, is_hash
-from streaming.base.index import get_index_basename
 from streaming.base.storage.upload import CloudUploader
+from streaming.base.util import bytes_to_int
 
 __all__ = ['JointWriter', 'SplitWriter']
 
@@ -42,8 +47,10 @@ class Writer(ABC):
             ``None``.
         hashes (List[str], optional): Optional list of hash algorithms to apply to shard files.
             Defaults to ``None``.
-        size_limit (int, optional): Optional shard size limit, after which point to start a new
-            shard. If ``None``, puts everything in one shard. Defaults to ``1 << 26``.
+        size_limit (Union[int, str], optional): Optional shard size limit, after which point
+            to start a new shard. If ``None``, puts everything in one shard. Can specify bytes
+            human-readable format as well, for example ``"100kb"`` for 100 kilobyte
+            (100*1024) and so on. Defaults to ``1 << 26``.
         extra_bytes_per_shard (int): Extra bytes per serialized shard (for computing shard size
             while writing). Defaults to ``0``.
         extra_bytes_per_sample (int): Extra bytes per serialized sample (for computing shard size
@@ -55,6 +62,8 @@ class Writer(ABC):
             max_workers (int): Maximum number of threads used to upload output dataset files in
                 parallel to a remote location. One thread is responsible for uploading one shard
                 file to a remote location. Default to ``min(32, (os.cpu_count() or 1) + 4)``.
+            retry (int): Number of times to retry uploading a file to a remote location.
+                Default to ``2``.
     """
 
     format: str = ''  # Name of the format (like "mds", "csv", "json", etc).
@@ -65,7 +74,7 @@ class Writer(ABC):
                  keep_local: bool = False,
                  compression: Optional[str] = None,
                  hashes: Optional[List[str]] = None,
-                 size_limit: Optional[int] = 1 << 26,
+                 size_limit: Optional[Union[int, str]] = 1 << 26,
                  extra_bytes_per_shard: int = 0,
                  extra_bytes_per_sample: int = 0,
                  **kwargs: Any) -> None:
@@ -73,25 +82,33 @@ class Writer(ABC):
         compression = compression or None
         if compression:
             if not is_compression(compression):
-                raise ValueError('Invalid compression: {compression}.')
+                raise ValueError(f'Invalid compression: {compression}.')
 
         hashes = hashes or []
         if list(hashes) != sorted(hashes):
             raise ValueError('Hashes must be unique and in sorted order.')
         for algo in hashes:
             if not is_hash(algo):
-                raise ValueError('Invalid hash: {algo}.')
+                raise ValueError(f'Invalid hash: {algo}.')
 
+        size_limit_value = None
         if size_limit:
-            if size_limit < 0:
-                raise ValueError('Size limit, if provided, must be greater than zero.')
-        else:
-            size_limit = None
+            size_limit_value = bytes_to_int(size_limit)
+            if size_limit_value < 0:
+                raise ValueError(f'`size_limit` must be greater than zero, instead, ' +
+                                 f'found as {size_limit_value}.')
+
+        # Validate keyword arguments
+        invalid_kwargs = [
+            arg for arg in kwargs.keys() if arg not in ('progress_bar', 'max_workers', 'retry')
+        ]
+        if invalid_kwargs:
+            raise ValueError(f'Invalid Writer argument(s): {invalid_kwargs} ')
 
         self.keep_local = keep_local
         self.compression = compression
         self.hashes = hashes
-        self.size_limit = size_limit
+        self.size_limit = size_limit_value
         self.extra_bytes_per_shard = extra_bytes_per_shard
         self.extra_bytes_per_sample = extra_bytes_per_sample
         self.new_samples: List[bytes]
@@ -99,12 +116,15 @@ class Writer(ABC):
 
         self.shards = []
 
-        self.cloud_writer = CloudUploader.get(out, keep_local, kwargs.get('progress_bar', False))
+        self.cloud_writer = CloudUploader.get(out, keep_local, kwargs.get('progress_bar', False),
+                                              kwargs.get('retry', 2))
         self.local = self.cloud_writer.local
         self.remote = self.cloud_writer.remote
         # `max_workers`: The maximum number of threads that can be executed in parallel.
         # One thread is responsible for uploading one shard file to a remote location.
         self.executor = ThreadPoolExecutor(max_workers=kwargs.get('max_workers', None))
+        # Create an event to track an exception in a thread.
+        self.event = Event()
 
         self._reset_cache()
 
@@ -219,6 +239,13 @@ class Writer(ABC):
         Args:
             sample (Dict[str, Any]): Sample dict.
         """
+        if self.event.is_set():
+            # Shutdown the executor and cancel all the pending futures due to exception in one of
+            # the threads.
+            self.cancel_future_jobs()
+            raise Exception('One of the threads failed. Check other traceback for more ' +
+                            'details.')
+        # Execute the task if there is no exception in any of the async threads.
         new_sample = self.encode_sample(sample)
         new_sample_size = len(new_sample) + self.extra_bytes_per_sample
         if self.size_limit and self.size_limit < self.new_shard_size + new_sample_size:
@@ -231,6 +258,11 @@ class Writer(ABC):
         """Write the index, having written all the shards."""
         if self.new_samples:
             raise RuntimeError('Internal error: not all samples have been written.')
+        if self.event.is_set():
+            # Shutdown the executor and cancel all the pending futures due to exception in one of
+            # the threads.
+            self.cancel_future_jobs()
+            return
         basename = get_index_basename()
         filename = os.path.join(self.local, basename)
         obj = {
@@ -239,7 +271,16 @@ class Writer(ABC):
         }
         with open(filename, 'w') as out:
             json.dump(obj, out, sort_keys=True)
-        self.executor.submit(self.cloud_writer.upload_file, basename)
+        # Execute the task if there is no exception in any of the async threads.
+        while self.executor._work_queue.qsize() > 0:
+            logger.debug(
+                f'Queue size: {self.executor._work_queue.qsize()}. Waiting for all ' +
+                f'shard files to get uploaded to {self.remote} before uploading index.json')
+            sleep(1)
+        logger.debug(f'Queue size: {self.executor._work_queue.qsize()}. Uploading ' +
+                     f'index.json to {self.remote}')
+        future = self.executor.submit(self.cloud_writer.upload_file, basename)
+        future.add_done_callback(self.exception_callback)
 
     def finish(self) -> None:
         """Finish writing samples."""
@@ -250,7 +291,36 @@ class Writer(ABC):
         logger.debug(f'Waiting for all shard files to get uploaded to {self.remote}')
         self.executor.shutdown(wait=True)
         if self.remote and not self.keep_local:
-            shutil.rmtree(self.local)
+            shutil.rmtree(self.local, ignore_errors=True)
+
+    def cancel_future_jobs(self) -> None:
+        """Shutting down the executor and cancel all the pending jobs."""
+        # Beginning python v3.9, ThreadPoolExecutor.shutdown() has a new parameter `cancel_futures`
+        if sys.version_info[1] <= 8:  # check if python version <=3.8
+            self.executor.shutdown(wait=False)
+        else:
+            self.executor.shutdown(wait=False, cancel_futures=True)
+        if self.remote and not self.keep_local:
+            shutil.rmtree(self.local, ignore_errors=True)
+
+    def exception_callback(self, future: Future) -> None:
+        """Raise an exception to the caller if exception generated by one of an async thread.
+
+        Also, set the thread event to let other threads knows about the exception in one of the
+        thread.
+
+        Args:
+            future (Future): Contains the status of the task
+
+        Raises:
+            exception: re-raise an exception
+        """
+        exception = future.exception()
+        if exception:
+            # Set the event to let other pool thread know about the exception
+            self.event.set()
+            # re-raise the same exception
+            raise exception
 
     def __enter__(self) -> Self:
         """Enter context manager.
@@ -269,6 +339,11 @@ class Writer(ABC):
             exc (BaseException, optional): Exc.
             traceback (TracebackType, optional): Traceback.
         """
+        if self.event.is_set():
+            # Shutdown the executor and cancel all the pending futures due to exception in one of
+            # the threads.
+            self.cancel_future_jobs()
+            return
         self.finish()
 
 
@@ -303,6 +378,8 @@ class JointWriter(Writer):
             max_workers (int): Maximum number of threads used to upload output dataset files in
                 parallel to a remote location. One thread is responsible for uploading one shard
                 file to a remote location. Default to ``min(32, (os.cpu_count() or 1) + 4)``.
+            retry (int): Number of times to retry uploading a file to a remote location.
+                Default to ``2``.
     """
 
     def __init__(self,
@@ -311,7 +388,7 @@ class JointWriter(Writer):
                  keep_local: bool = False,
                  compression: Optional[str] = None,
                  hashes: Optional[List[str]] = None,
-                 size_limit: Optional[int] = 1 << 26,
+                 size_limit: Optional[Union[int, str]] = 1 << 26,
                  extra_bytes_per_shard: int = 0,
                  extra_bytes_per_sample: int = 0,
                  **kwargs: Any) -> None:
@@ -334,6 +411,12 @@ class JointWriter(Writer):
         raise NotImplementedError
 
     def flush_shard(self) -> None:
+        if self.event.is_set():
+            # Shutdown the executor and cancel all the pending futures due to exception in one of
+            # the threads.
+            self.cancel_future_jobs()
+            return
+
         raw_data_basename, zip_data_basename = self._name_next_shard()
         raw_data = self.encode_joint_shard()
         raw_data_info, zip_data_info = self._process_file(raw_data, raw_data_basename,
@@ -346,7 +429,10 @@ class JointWriter(Writer):
         obj.update(self.get_config())
         self.shards.append(obj)
 
-        self.executor.submit(self.cloud_writer.upload_file, zip_data_basename or raw_data_basename)
+        # Execute the task if there is no exception in any of the async threads.
+        future = self.executor.submit(self.cloud_writer.upload_file, zip_data_basename or
+                                      raw_data_basename)
+        future.add_done_callback(self.exception_callback)
 
 
 class SplitWriter(Writer):
@@ -378,6 +464,8 @@ class SplitWriter(Writer):
             max_workers (int): Maximum number of threads used to upload output dataset files in
                 parallel to a remote location. One thread is responsible for uploading one shard
                 file to a remote location. Default to ``min(32, (os.cpu_count() or 1) + 4)``.
+            retry (int): Number of times to retry uploading a file to a remote location.
+                Default to ``2``.
     """
 
     extra_bytes_per_shard = 0
@@ -389,7 +477,7 @@ class SplitWriter(Writer):
                  keep_local: bool = False,
                  compression: Optional[str] = None,
                  hashes: Optional[List[str]] = None,
-                 size_limit: Optional[int] = 1 << 26,
+                 size_limit: Optional[Union[int, str]] = 1 << 26,
                  **kwargs: Any) -> None:
         super().__init__(out=out,
                          keep_local=keep_local,
@@ -410,6 +498,12 @@ class SplitWriter(Writer):
         raise NotImplementedError
 
     def flush_shard(self) -> None:
+        if self.event.is_set():
+            # Shutdown the executor and cancel all the pending futures due to exception in one of
+            # the threads.
+            self.cancel_future_jobs()
+            return
+
         raw_data_basename, zip_data_basename = self._name_next_shard()
         raw_meta_basename, zip_meta_basename = self._name_next_shard('meta')
         raw_data, raw_meta = self.encode_split_shard()
@@ -427,5 +521,12 @@ class SplitWriter(Writer):
         obj.update(self.get_config())
         self.shards.append(obj)
 
-        self.executor.submit(self.cloud_writer.upload_file, zip_data_basename or raw_data_basename)
-        self.executor.submit(self.cloud_writer.upload_file, zip_meta_basename or raw_meta_basename)
+        # Execute the task if there is no exception in any of the async threads.
+        future = self.executor.submit(self.cloud_writer.upload_file, zip_data_basename or
+                                      raw_data_basename)
+        future.add_done_callback(self.exception_callback)
+
+        # Execute the task if there is no exception in any of the async threads.
+        future = self.executor.submit(self.cloud_writer.upload_file, zip_meta_basename or
+                                      raw_meta_basename)
+        future.add_done_callback(self.exception_callback)
