@@ -5,7 +5,7 @@
 
 import os
 from copy import deepcopy
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Union
 
 import numpy as np
 from typing_extensions import Self
@@ -31,8 +31,10 @@ class XSVReader(SplitReader):
         raw_meta (FileInfo): Uncompressed meta file info.
         samples (int): Number of samples in this shard.
         separator (str): Separator character(s).
-        size_limit (int, optional): Optional shard size limit, after which point to start a new
-            shard. If None, puts everything in one shard.
+        size_limit (Union[int, str], optional): Optional shard size limit, after
+            which point to start a new shard. If None, puts everything in one shard.
+            Can specify bytes in human-readable format as well, for example
+            ``"100kb"`` for 100 kilobyte (100*1024) and so on.
         zip_data (FileInfo, optional): Compressed data file info.
         zip_meta (FileInfo, optional): Compressed meta file info.
     """
@@ -50,7 +52,7 @@ class XSVReader(SplitReader):
         raw_meta: FileInfo,
         samples: int,
         separator: str,
-        size_limit: Optional[int],
+        size_limit: Optional[Union[int, str]],
         zip_data: Optional[FileInfo],
         zip_meta: Optional[FileInfo],
     ) -> None:
@@ -74,9 +76,13 @@ class XSVReader(SplitReader):
             Self: Loaded XSVReader.
         """
         args = deepcopy(obj)
-        assert args['version'] == 2
+        if args['version'] != 2:
+            raise ValueError(f'Unsupported streaming data version: {args["version"]}. ' +
+                             f'Expected version 2.')
         del args['version']
-        assert args['format'] == 'xsv'
+        if args['format'] != 'xsv':
+            raise ValueError(f'Unsupported data format: {args["format"]}. ' +
+                             f'Expected to be `xsv`.')
         del args['format']
         args['dirname'] = dirname
         args['split'] = split
@@ -179,9 +185,13 @@ class CSVReader(XSVReader):
             Self: Loaded CSVReader.
         """
         args = deepcopy(obj)
-        assert args['version'] == 2
+        if args['version'] != 2:
+            raise ValueError(f'Unsupported streaming data version: {args["version"]}. ' +
+                             f'Expected version 2.')
         del args['version']
-        assert args['format'] == 'csv'
+        if args['format'] != 'csv':
+            raise ValueError(f'Unsupported data format: {args["format"]}. ' +
+                             f'Expected to be `csv`.')
         del args['format']
         args['dirname'] = dirname
         args['split'] = split
@@ -246,9 +256,13 @@ class TSVReader(XSVReader):
             Self: Loaded TSVReader.
         """
         args = deepcopy(obj)
-        assert args['version'] == 2
+        if args['version'] != 2:
+            raise ValueError(f'Unsupported streaming data version: {args["version"]}. ' +
+                             f'Expected version 2.')
         del args['version']
-        assert args['format'] == 'tsv'
+        if args['format'] != 'tsv':
+            raise ValueError(f'Unsupported data format: {args["format"]}. ' +
+                             f'Expected to be `tsv`.')
         del args['format']
         args['dirname'] = dirname
         args['split'] = split
