@@ -2,26 +2,25 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import multiprocessing as mp
+import os
 import re
 from multiprocessing.managers import ListProxy
 from random import random
 from time import sleep
 from typing import Any
 
-import numpy as np
 import pytest
 
-from streaming.base.shared import SharedBarrier
+from streaming.base.shared import SharedArray, SharedBarrier
 
 
 class TestSharedBarrier:
 
-    @pytest.mark.parametrize('filelock_path', ['/tmp/dir/file_path'])
-    @pytest.mark.parametrize('shm_path', ['barrier_shm_path'])
-    def test_params(self, filelock_path: str, shm_path: str):
-        barrier = SharedBarrier(filelock_path, shm_path)
-        assert barrier.filelock_path == filelock_path
-        assert isinstance(barrier._arr, np.ndarray)
+    @pytest.mark.parametrize('filelock_path', ['barrier_filelock_path'])
+    @pytest.mark.parametrize('shm_name', ['barrier_shm_name'])
+    def test_params(self, filelock_path: str, shm_name: str):
+        barrier = SharedBarrier(filelock_path, shm_name)
+        assert isinstance(barrier._arr, SharedArray)
         assert barrier._arr.shape == (3,)
         assert barrier.num_enter == 0
         assert barrier.num_exit == -1
@@ -31,7 +30,7 @@ class TestSharedBarrier:
     @pytest.mark.parametrize('num_exit', [4, 9])
     @pytest.mark.parametrize('flag', [True, False])
     def test_setter_getter(self, num_enter: int, num_exit: int, flag: bool):
-        barrier = SharedBarrier('/tmp/dir/file_path', 'barrier_shm_path')
+        barrier = SharedBarrier('/tmp/dir/filelock_path', 'barrier_shm_name')
         barrier.num_enter = num_enter
         assert barrier.num_enter == num_enter
         barrier.num_exit = num_exit
@@ -48,11 +47,13 @@ class TestSharedBarrier:
         shared_list.append(f'passed barrier again: {mp.current_process().name}')
 
     @pytest.mark.parametrize('num_process', [2, 3])
-    def test_barrier(self, num_process: int):
+    @pytest.mark.parametrize('filelock_root', ['/tmp/dir/'])
+    def test_barrier(self, num_process: int, filelock_root: str):
         mp.set_start_method('fork', force=True)
         manager = mp.Manager()
         shared_list = manager.list()
-        barrier = SharedBarrier('/tmp/dir/file_path', 'barrier_shm_path')
+        os.makedirs(filelock_root, exist_ok=True)
+        barrier = SharedBarrier(os.path.join(filelock_root, 'filelock_path'), 'barrier_shm_name')
         processes = [
             mp.Process(target=self.run, args=(num_process, barrier, shared_list))
             for _ in range(num_process)
