@@ -3,7 +3,8 @@
 
 import json
 import tempfile
-from typing import Any, Union
+from decimal import Decimal
+from typing import Any, Tuple, Union
 
 import numpy as np
 import pytest
@@ -75,6 +76,62 @@ class TestMDSEncodings:
             int_enc = mdsEnc.Int()
             _ = int_enc.encode(data)
 
+    @pytest.mark.parametrize('dtype_str', [
+        'uint8',
+        'uint16',
+        'uint32',
+        'uint64',
+        'int8',
+        'int16',
+        'int32',
+        'int64',
+        'float16',
+        'float32',
+        'float64',
+    ])
+    @pytest.mark.parametrize('shape', [
+        (1,),
+        (1, 1),
+        (2, 3, 4, 5),
+        (1, 3, 1, 3, 1),
+        (300,),
+        (1, 256, 1),
+        (65536, 7, 1),
+    ])
+    def test_ndarray_encode_decode(self, dtype_str: str, shape: Tuple[int]):
+        dtype = getattr(np, dtype_str)
+        a = np.random.randint(0, 1000, shape).astype(dtype)
+
+        encoding = 'ndarray'
+        assert mdsEnc.is_mds_encoding(encoding)
+        assert mdsEnc.get_mds_encoded_size(encoding) is None
+        b = mdsEnc.mds_encode(encoding, a)
+        c = mdsEnc.mds_decode(encoding, b)
+        assert (a == c).all()
+        b1_len = len(b)
+
+        encoding = f'ndarray:{dtype.__name__}'
+        assert mdsEnc.is_mds_encoding(encoding)
+        assert mdsEnc.get_mds_encoded_size(encoding) is None
+        b = mdsEnc.mds_encode(encoding, a)
+        c = mdsEnc.mds_decode(encoding, b)
+        assert (a == c).all()
+        b2_len = len(b)
+
+        shape_str = ','.join(map(str, shape))
+        encoding = f'ndarray:{dtype.__name__}:{shape_str}'
+        assert mdsEnc.is_mds_encoding(encoding)
+        b_size = mdsEnc.get_mds_encoded_size(encoding)
+        assert b_size is not None
+        b = mdsEnc.mds_encode(encoding, a)
+        c = mdsEnc.mds_decode(encoding, b)
+        assert (a == c).all()
+        assert len(b) == b_size
+        b3_len = len(b)
+
+        assert b3_len < b2_len < b1_len
+        assert b3_len == np.prod(shape) * dtype().nbytes
+
     @pytest.mark.parametrize('mode', ['I', 'L', 'RGB'])
     def test_pil_encode_decode(self, mode: str):
         pil_enc = mdsEnc.PIL()
@@ -364,12 +421,48 @@ class TestMDSEncodings:
         assert isinstance(dec, np.floating)
         assert dec == decoded
 
+    @pytest.mark.parametrize(('decoded', 'encoded'), [(42, b'42'), (-42, b'-42')])
+    def test_mds_StrInt(self, decoded: int, encoded: bytes):
+        coder = mdsEnc.StrInt()
+        enc = coder.encode(decoded)
+        assert isinstance(enc, bytes)
+        assert enc == encoded
+
+        dec = coder.decode(encoded)
+        assert isinstance(dec, int)
+        assert dec == decoded
+
+    @pytest.mark.parametrize(('decoded', 'encoded'), [(42.0, b'42.0'), (-42.0, b'-42.0')])
+    def test_mds_StrFloat(self, decoded: float, encoded: bytes):
+        coder = mdsEnc.StrFloat()
+        enc = coder.encode(decoded)
+        assert isinstance(enc, bytes)
+        assert enc == encoded
+
+        dec = coder.decode(encoded)
+        assert isinstance(dec, float)
+        assert dec == decoded
+
+    @pytest.mark.parametrize(('decoded', 'encoded'), [(Decimal('4E15'), b'4E+15'),
+                                                      (Decimal('-4E15'), b'-4E+15')])
+    def test_mds_StrDecimal(self, decoded: Decimal, encoded: bytes):
+        coder = mdsEnc.StrDecimal()
+        enc = coder.encode(decoded)
+        assert isinstance(enc, bytes)
+        assert enc == encoded
+
+        dec = coder.decode(encoded)
+        assert isinstance(dec, Decimal)
+        assert dec == decoded
+
     def test_get_mds_encodings(self):
         uints = {'uint8', 'uint16', 'uint32', 'uint64'}
-        ints = {'int8', 'int16', 'int32', 'int64'}
-        floats = {'float16', 'float32', 'float64'}
+        ints = {'int8', 'int16', 'int32', 'int64', 'str_int'}
+        floats = {'float16', 'float32', 'float64', 'str_float', 'str_decimal'}
         scalars = uints | ints | floats
-        expected_encodings = {'int', 'bytes', 'json', 'png', 'jpeg', 'str', 'pil', 'pkl'} | scalars
+        expected_encodings = {
+            'int', 'bytes', 'json', 'ndarray', 'png', 'jpeg', 'str', 'pil', 'pkl'
+        } | scalars
         enc = mdsEnc.get_mds_encodings()
         assert len(enc) == len(expected_encodings)
         assert enc == expected_encodings
