@@ -18,6 +18,13 @@ logger = logging.getLogger(__name__)
 
 class TestMDSWriter:
 
+    def test_invalid_args(self, local_remote_dir: Tuple[str, str]):
+        local, _ = local_remote_dir
+        dataset = SequenceDataset(100)
+        columns = dict(zip(dataset.column_names, dataset.column_encodings))
+        with pytest.raises(ValueError, match=f'.*Invalid Writer argument.*'):
+            _ = MDSWriter(out=local, columns=columns, min_workers=1)
+
     @pytest.mark.parametrize('num_samples', [100])
     @pytest.mark.parametrize('size_limit', [32])
     def test_config(self, local_remote_dir: Tuple[str, str], num_samples: int,
